part of tags;

final Tag editableLabel = createTag((_) => new EditableLabel());

class EditableLabel extends HtmlElement {
 
  static const TAG = 'editable-label';
 
  ShadowRoot _shadow;
  LabelElement _label;
  TextInputElement _textInput;
  
  String _text;
  String _lastText;
  
  Function displayFilter;
  
  bool _isEditing = false;
  /// Whether or not this editable label is editing.
  bool get isEditing => _isEditing;
  
  final _onTextChange = new StreamController<String>.broadcast();
  Stream<String> get onTextChange => _onTextChange.stream;
  
  final _onEditChange = new StreamController<bool>.broadcast();
  /// An observable stream of this editable label's state transitions.
  /// 
  /// A `true` value represents a transition to an [isEditing] value of `true`.
  Stream<bool> get onEditChange => _onEditChange.stream;
  
  String 
    get text => _text;
    set text(String t) {
      if (_textInput.value != t) {
        _textInput.value = t;
      }
      String displayText = t;
      if (displayFilter != null) {
        // TODO(rms): usage of explicit .call() is a workaround for:
        // https://code.google.com/p/dart/issues/detail?id=15401
        displayText = displayFilter.call(t);
      }
      if (_label.text != displayText) {
        _label.text = displayText;
      }
      if (_text != t) {
        _text = t;
        _onTextChange.add(t);
      }
    }  
  
  factory EditableLabel() => new Element.tag(TAG);
  
  EditableLabel.created() : super.created() {        
    children(this, () {
      _shadow = shadow(() {
        _label = label();
      })
      ..applyAuthorStyles = true;
    });
    _textInput = new InputElement()
    ..onBlur.listen((_) {      
      cancelEdit();
    })
    ..onDoubleClick.listen((Event ev) {
      ev.stopPropagation();      
    })
    ..onKeyUp.listen((KeyboardEvent ev) {
      if (ev.keyCode == KeyCode.ESC) {        
        cancelEdit();
      } else if (ev.keyCode == KeyCode.ENTER) {
        completeEdit();
      }
    })
    ..type = 'text';
  }
  
  void beginEdit() {
    if (!isEditing) {
      _lastText = this.text;    
      _shadow.children[0] = _textInput;
      _textInput.focus();
      _onEditChange.add(_isEditing = true);
    }
  }
  
  void cancelEdit() {
    if (isEditing) {
      this.text = _lastText;
      _shadow.children[0] = _label;
      _onEditChange.add(_isEditing = false);
    }
  }
  
  void completeEdit() {
    if (isEditing) {
      this.text = _textInput.value;
      _lastText = this.text;
      _shadow.children[0] = _label;
      _onEditChange.add(_isEditing = false);
    }
  }
}
